/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvgui.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvgui.h>

QVGUI::QVGUI(QWidget *parent): QWidget(parent),qvimagecanvaslist()
	{
	qDebug() << "QVGUI::QVGUI()";

	if(qvApp == NULL)
		{
		QString str = "QVGUI::QVGUI(): "
						": the QVGUI cannot be created before the "
						"QVApplication instance. Aborting now.";
		std::cerr << qPrintable(str) << std::endl;
		exit(1);
		}

	connect(this,SIGNAL(closed()),qvApp,SLOT(quit()));

	hboxlayout = new QHBoxLayout();
	leftwidget = new QWidget();
	qgroupbox = new QGroupBox("Workers input");
	leftvboxlayout = new QVBoxLayout(leftwidget);
	toolbox = new QToolBox(this);

	setWindowTitle("QVGUI");

	QSetIterator<QVPropertyContainer *> iw(qvApp->getQVPropertyContainers());
	while (iw.hasNext())
		{
		QVPropertyContainer* qvp = iw.next();
		QVWorker* worker;
		if((worker = dynamic_cast<QVWorker*>(qvp)) != NULL)
			{
			toolbox->addItem(new QVParamsInspectorWidget(worker),worker->getName());
			// Adding worker interface small
			QVWorkerInterfaceSmall *workerWidget = new QVWorkerInterfaceSmall(worker);
			leftvboxlayout->addWidget(workerWidget);
			}
		}
	leftvboxlayout->addStretch();
	iw.toFront();
	while (iw.hasNext())
		{
		QVPropertyContainer* qvp = iw.next();
		QVCamera* cam;
		if((cam = dynamic_cast<QVCamera*>(qvp)) != NULL)
			{
			QVCameraWidgetSmall *cameraWidget = new QVCameraWidgetSmall(cam);
			leftvboxlayout->addWidget(cameraWidget);
			}
		}

	qvApp->registerGUI(this);

	leftvboxlayout->setSpacing(0);
	leftvboxlayout->setMargin(0);
	hboxlayout->addWidget(leftwidget);
	groupboxlayout = new QVBoxLayout(qgroupbox);
	groupboxlayout->addWidget(toolbox);
	hboxlayout->addWidget(qgroupbox);
	setLayout(hboxlayout);

	show();
	qDebug() << "QVGUI::QVGUI() <~ return";
	}

QVGUI::~QVGUI()
	{
	QListIterator<QVImageCanvas*> i(qvimagecanvaslist);
	while (i.hasNext())
	delete(i.next());
	}

void QVGUI::closeEvent(QCloseEvent *event)
	{
	Q_UNUSED(event);
	emit closed();
	}

void QVGUI::add(QVCamera *camera)
	{
	QVCameraWidgetSmall *cameraWidget = new QVCameraWidgetSmall(camera);
	leftvboxlayout->addWidget(cameraWidget);
	}

void QVGUI::add(QVWorker * worker)
	{
	// Adding params
	QVParamsInspectorWidget *qvparamsinspectorwidget;
	qvparamsinspectorwidget = new QVParamsInspectorWidget(worker,this);
	toolbox->addItem(qvparamsinspectorwidget,worker->getName());
	qvparamsinspectorwidget->updateGeometry();

	// Adding worker interface small
	QVWorkerInterfaceSmall *workerWidget = new QVWorkerInterfaceSmall(worker);
	leftvboxlayout->addWidget(workerWidget);
	}
